function contours = getContour(mat, width, height)
	% GETCONTOUR Return the positions of the shape's contours. The
	% positions are given in a gds-compatible format meaning that the
	% contour of holes are linked to the exterior contours of the shape in
	% which they are nucleated
	%
	% Arguments:
	%     mat - (matrix) The shape is given by the 0 level-set contour of mat.
  %   width - (int) Width of the structure in nm.
  %  height - (int) Height of the structure in nm.
	%
	% Outputs:
	%   contours - (cell array) each value in the array contain the
	%   position of one contour of the shape and is of the form:
	%     x - (double) Nodes x position
	%     y - (double) Nodes y position
	%     nb - (int) Number of nodes
	%
	% Example:
	%   C = getContour([-1,-1;1,-1],.8,.25)
	%   hold all;
	%	  for i = 1:length(C)
	%      plot(C(i).x, C(i).y);
	%   end
	%
  % @author Nicolas Lebbe
  % @date 22/04/2020

	X = linspace(0, width, size(mat,1))';
	Y = linspace(0, height, size(mat,2))';

	% Add exterior contour
	dx = X(2) - X(1);
	dy = Y(2) - Y(1);
	XX = [-dx; X; width+dx];
	YY = [-dy; Y; height+dy];
	phi = 1e30*ones(size(mat)+2);
	phi(2:end-1,2:end-1) = mat;

	% Find contour
	cc = contourc(YY, XX, phi, [0,0]);

	% Determine number of contours
	i = 1;
	nbc = 0;
	while i < size(cc,2)
		i = i + cc(2,i) + 1;
		nbc = nbc+1;
	end
	contours = cell(1,nbc);

	% Define contours structure
	i = 1;
	id = 1;
	while i < size(cc,2)
		nb = cc(2,i);
		xs = cc(2,i+1:i+nb);
		ys = cc(1,i+1:i+nb);

		% remove same consecutive values
		while ~isempty(xs)
			eqx = abs([diff(xs),xs(1)-xs(end)]) < 1e-9;
			eqy = abs([diff(ys),ys(1)-ys(end)]) < 1e-9;
			eq = eqx & eqy;
			if sum(eq) == 0
				break;
			end
			xs(eq) = [];
			ys(eq) = [];
		end

		if ~isempty(xs)

            xs = max(min(xs, width), 0);
            ys = max(min(ys, height), 0);

			% Always anticlockwise order
			clockwise = sum( ...
				( xs([2:end,1]) - xs ) .* ...
				( ys([2:end,1]) + ys ) ...
			);
			if clockwise >= 0
				xs = xs(end:-1:1);
				ys = ys(end:-1:1);
			end

			contours{id}.x = xs;
			contours{id}.y = ys;
			contours{id}.nb = numel(xs);
			contours{id}.id = id;
			contours{id}.inside = zeros(1,nbc);

			id = id + 1;

		end

		i = i + nb + 1;
	end
	nbc = id - 1;

	% Test if inside another polygon
	for i = 1:nbc
		xq = contours{i}.x(1);
		yq = contours{i}.y(1);
		for j = 1:nbc
			if i == j, continue; end
			x = contours{j}.x;
			y = contours{j}.y;
			inp = inpolygon(xq,yq,x,y);
			if inp
				contours{i}.inside(j) = true;
			end
		end
		% Interior contour if polygon
		% inside odd number of polygons
		contours{i}.nbinside = sum(contours{i}.inside);
		contours{i}.interior = mod(contours{i}.nbinside,2);
	end

	% Fusion of interior and exterior contours
	for i = 1:nbc
		if contours{i}.interior
			nbin = contours{i}.nbinside;

			% Find direct parent
			parent = 0;
            for j = 1:nbc
                if contours{i}.inside(j)
                    if contours{j}.nbinside == nbin-1
                        parent = j;
                        break;
                    end
                end
            end

			% Find closest points for fusion
			x1 = contours{parent}.x;
			y1 = contours{parent}.y;
			n1 = contours{parent}.nb;
			x2 = contours{i}.x;
			y2 = contours{i}.y;
            n2 = contours{i}.nb;

			X1 = repmat(x1,n2,1);
			Y1 = repmat(y1,n2,1);
			X2 = repmat(x2',1,n1);
			Y2 = repmat(y2',1,n1);
			dist = (X1-X2).^2 + (Y1-Y2).^2;
			[~,ind] = min(dist(:));
			[i2,i1] = ind2sub([n2,n1],ind);

			contours{parent}.x = [x1(1:i1), x2(i2:-1:1), ...
				x2(end:-1:i2), x1(i1:end)];
			contours{parent}.y = [y1(1:i1), y2(i2:-1:1), ...
				y2(end:-1:i2), y1(i1:end)];

			contours{parent}.nb = n1+n2+2;
		end
	end

	% Keep only exteriors
	interiors = ~[cellfun(@(x)x.interior,contours)];
	contours = [contours{interiors}];

end
