function [] = toGDS(mat, filename, varargin)
  % TOGDS Export a matrix seen as a level-set function into a GDSII file
  %
  % Arguments:
  %        mat - (matrix) The shape is given by a level-set contour of mat.
  %   filename - (string) Filename for the generated .gds file.
  %   varargin - optional additional parameters
  %     [width=1000] - (int) Width of the structure in nm.
  %    [height=1000] - (int) Height of the structure in nm.
  %     [levelset=x] - (float) Level set value used to construct the contour
  %                  -> The default value x is the mean value of mat.
  %        [layer=2] - (string) Layer name.
  %     [datatype=0] - (string) Datatype name.
  %
  % @author Nicolas Lebbe
  % @date 22/04/2020

  % Default parameters
  width = 1000;
  height = 1000;
  levelset = mean(mat(:));
	layer = 2;
	datatype = 0;

  % Read optional parameters
	for i = 3:2:(nargin-1)
    parameter = varargin{i-2};
    value = varargin{i-1};
		switch parameter
			case 'layer',    layer    = value;
			case 'datatype', datatype = value;
			case 'width',    width    = value;
			case 'height',   height   = value;
			case 'levelset', levelset = value;
		end
	end

  % Get contour of the structure
  B = getContour(mat - levelset, width, height);

  % Write the GDS file in ASCII format
  file = fopen(filename, 'w');

  % HEADER record
  fwrite(file, 1*2+1*2+1*2, 'uint16', 'b'); % (size)
  fwrite(file, 0x0002, 'uint16', 'b'); % HEADER
  fwrite(file, 600, 'uint16', 'b'); % (version)

  % BGNLIB record
  fwrite(file, 1*2+1*2+12*2, 'uint16', 'b'); % (size)
  fwrite(file, 0x0102, 'uint16', 'b'); % BGNLIB
  for i = 1:12
    fwrite(file, 0x0000, 'uint16', 'b'); % last modification
  end

  % LIBNAME record
  fwrite(file, 1*2+1*2+4*1, 'uint16', 'b'); % (size)
  fwrite(file, 0x0206, 'uint16', 'b'); % LIBNAME
  fwrite(file, 'NANO', 'char', 'b'); % (name of the library)

  % UNITS record
  fwrite(file, 1*2+1*2+2*8, 'uint16', 'b'); % (size)
  fwrite(file, 0x0305, 'uint16', 'b'); % UNITS
  % values here are 8-bytes float in Excess-64 format for exponent
  fwrite(file, [0x3e41,0x8937,0x4bc6,0xa7f0], 'uint16', 'b'); % (uunit 0.001 um)
  fwrite(file, [0x3944,0xb82f,0xa09b,0x5a54], 'uint16', 'b'); % (dunit 1e-9 nm)

  % BGNSTR record
  fwrite(file, 1*2+1*2+12*2, 'uint16', 'b'); % (size)
  fwrite(file, 0x0502, 'uint16', 'b'); % BGNSTR
  for i = 1:12
    fwrite(file, 0x0000, 'uint16', 'b'); % last modification
  end

  % STRNAME record
  fwrite(file, 1*2+1*2+4*1, 'uint16', 'b'); % (size)
  fwrite(file, 0x0606, 'uint16', 'b'); % STRNAME
  fwrite(file, 'COMP', 'char', 'b'); % (name of the component)

  for i = 1:length(B)
      % BOUNDARY record
      fwrite(file, 1*2+1*2, 'uint16', 'b'); % (size)
      fwrite(file, 0x0800, 'uint16', 'b'); % BOUNDARY

      % LAYER record
      fwrite(file, 1*2+1*2+1*2, 'uint16', 'b'); % (size)
      fwrite(file, 0x0D02, 'uint16', 'b'); % LAYER
      fwrite(file, layer, 'uint16', 'b'); % (layer)

      % DATATYPE record
      fwrite(file, 1*2+1*2+1*2, 'uint16', 'b'); % (size)
      fwrite(file, 0x0E02, 'uint16', 'b'); % DATATYPE
      fwrite(file, datatype, 'uint16', 'b'); % (datatype)

      % XY record
      nb = B(i).nb*2;
      fwrite(file, 1*2+1*2+nb*4, 'uint16', 'b'); % (size)
      fwrite(file, 0x1003, 'uint16', 'b'); % XY
      fwrite(file, [B(i).x; B(i).y], 'int32', 'b'); % (positions)

      % ENDEL record
      fwrite(file, 1*2+1*2, 'uint16', 'b'); % (size)
      fwrite(file, 0x1100, 'uint16', 'b'); % ENDEL
  end

  % ENDSTR record
  fwrite(file, 1*2+1*2, 'uint16', 'b'); % (size)
  fwrite(file, 0x0700, 'uint16', 'b'); % ENDSTR

  % ENDLIB record
  fwrite(file, 1*2+1*2, 'uint16', 'b'); % (size)
  fwrite(file, 0x0400, 'uint16', 'b'); % ENDLIB

  fclose(file);

end
